<?php

namespace Xfrocks\AttachmentStore\Job;

use XF\Job\AbstractJob;

class DeleteUnusedData extends AbstractJob
{
    /**
     * @var array
     */
    protected $defaultData = [
        'start' => 0,
        'batch' => 100,
    ];

    public function run($maxRunTime)
    {
        $startTime = microtime(true);

        $attachments = $this->app->finder('XF:AttachmentData')
            ->where('attach_count', 0)
            ->fetch($this->data['batch']);

        $done = 0;

        /** @var \XF\Entity\AttachmentData $attachment */
        foreach ($attachments as $attachment) {
            $this->data['start'] = $attachment->data_id;

            try {
                $attachment->delete();
            } catch (\Exception $e) {
                // ignore
            }
            $done++;

            if (microtime(true) - $startTime >= $maxRunTime) {
                break;
            }
        }

        if ($done === 0) {
            return $this->complete();
        }

        $this->data['batch'] = $this->calculateOptimalBatch($this->data['batch'], $done, $startTime, $maxRunTime, 1000);

        return $this->resume();
    }

    /**
     * @return string
     */
    public function getStatusMessage()
    {
        $actionPhrase = \XF::phrase('deleting');
        $typePhrase = \XF::phrase('attachments');
        return sprintf('%s... %s (%s)', $actionPhrase, $typePhrase, $this->data['start']);
    }

    /**
     * @return bool
     */
    public function canCancel()
    {
        return true;
    }

    /**
     * @return bool
     */
    public function canTriggerByChoice()
    {
        return true;
    }
}
