## Use S3-compatible storage

### Digital Ocean Spaces

```php
$config['bdAttachmentStore_s3Buckets'] = array(
    'space-name' => array(
        'key' => 'foo',
        'secret' => 'bar',

        // the configured CDN domain or the default URL below Space name
        'url' => 'https://space-name.region.digitaloceanspaces.com',

        // look for Endpoint in Settings tab
        'endpoint' => 'https://region.digitaloceanspaces.com',
    ),
);
```

## Use the same configured storage for /data

```php
$config['fsAdapters']['data'] = function () {
    return \Xfrocks\AttachmentStore\Fs::getAdapterDefault();
};
$config['externalDataUrl'] = function ($externalPath, $canonical) {
    return 'https://s3.amazonaws.com/bucket/' . $externalPath;
};
```

## Advanced usage: custom adapter for /data

```php
$config['fsAdapters']['data'] = function () {
    return \Xfrocks\AttachmentStore\Fs::getAdapterCustom(
        's3',
        [
            'bucket' => 'bucket',
            'key' => 'foo',
            'secret' => 'bar',
        ]
    );
};

$config['externalDataUrl'] = function ($externalPath, $canonical) {
    return 'https://s3.amazonaws.com/bucket/' . $externalPath;
};
```

## Advanced usage: file_path whitelist

By default, this add-on will ignore attachments with custom file_path because they are quite special
(the uploader most likely will do something fancy with the files, so it's safer to leave them in the intended location).
We maintain a default whitelist to handle the most common cases:

- XF video attachment

If you have other add-ons that make use of the custom file_path and you want to upload those files to remote storage, 
you can add these into `config.php`:

```php
$config['bdAttachmentStore_filePathWhitelist'] = [
    // XF2 video attachment files:
    'data://video/%FLOOR%/%DATA_ID%-%HASH%.mp4',
    // custom add-on:
    'data://vendor/addOnId/%DATA_ID%.xxx',
];
```

## Advanced usage: custom MIME types

This add-on uses a library to detect MIME type from a file extension, it has most of the common file types.
If you want to override the default or support new extension, add something like this in `config.php`:

```php
$config['bdAttachmentStore_mimeTypes'] = [
    'apk' => 'application/vnd.android.package-archive',
];
```
