<?php

namespace Xfrocks\AttachmentStore;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Column;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
    use StepRunnerInstallTrait;
    use StepRunnerUpgradeTrait;
    use StepRunnerUninstallTrait;

    public function installStep1()
    {
        /** @noinspection DuplicatedCode */
        $sm = $this->schemaManager();

        foreach ($this->getTables() as $tableName => $closure) {
            $sm->createTable($tableName, $closure);
        }

        foreach ($this->getColumns() as $tableName => $tableColumns) {
            $sm->alterTable($tableName, function (Alter $table) use ($tableColumns) {
                foreach ($tableColumns as $columnName => $closure) {
                    $column = $table->addColumn($columnName);
                    $closure($column);
                }
            });
        }
    }

    public function uninstallStep1()
    {
        $sm = $this->schemaManager();

        foreach (array_keys($this->getTables()) as $tableName) {
            $sm->dropTable($tableName);
        }

        foreach ($this->getColumns() as $tableName => $tableColumns) {
            $sm->alterTable($tableName, function (Alter $table) use ($tableColumns) {
                $table->dropColumns(array_keys($tableColumns));
            });
        }
    }

    /**
     * @return array
     */
    private function getTables()
    {
        $tables = [];

        $tables['xf_bdattachmentstore_delay'] = function (Create $table) {
            $table->addColumn('data_id', 'int');
            $table->addColumn('insert_date', 'int');
            $table->addKey('insert_date');
        };

        return $tables;
    }

    /**
     * @return array
     */
    private function getColumns()
    {
        $columns = [];
        $columns['xf_attachment_data'] = [];
        $columns['xf_attachment_data']['bdattachmentstore_engine'] = function (Column $column) {
            $column->type('varchar', 100)->nullable(true);
        };
        $columns['xf_attachment_data']['bdattachmentstore_options'] = function (Column $column) {
            $column->type('mediumblob')->nullable(true);
        };

        return $columns;
    }
}
