<?php

namespace Xfrocks\AttachmentStore\Util;

use XF\Util\File as FileUtil;

class MaintainFilename
{
    /**
     * @param array $engineOptions
     * @param string $filePath
     * @return string
     */
    public static function getFilePath($engineOptions, $filePath)
    {
        if (!isset($engineOptions['filename']) || $engineOptions['filename'] === '') {
            return $filePath;
        }

        $dirName = dirname($filePath);
        $basename = basename($filePath);
        $extension = FileUtil::getFileExtension($basename);
        if ($extension === '') {
            $basenameWithoutExt = $basename;
        } else {
            $basenameWithoutExt = utf8_substr($basename, 0, intval(-1 * (utf8_strlen($extension) + 1)));
        }
        $basenameParts = explode('_', $basenameWithoutExt);

        if (is_numeric($basenameParts[0])) {
            if (!isset($engineOptions['filenameFormat']) || $engineOptions['filenameFormat'] === '') {
                $fileName = sprintf('%d_%s', $basenameParts[0], $engineOptions['filename']);
            } else {
                $variables = [
                    '{data_id}' => $basenameParts[0],
                    '{file_hash}' => $basenameParts[1],
                    '{extension}' => FileUtil::getFileExtension($engineOptions['filename']),
                ];

                if ($variables['{extension}'] === '') {
                    $variables['{name}'] = $engineOptions['filename'];
                } else {
                    $variables['{name}'] = utf8_substr(
                        $engineOptions['filename'],
                        0,
                        intval(-1 * (utf8_strlen($variables['{extension}']) + 1))
                    );
                }

                $fileName = $engineOptions['filenameFormat'];

                $fileName = strval(str_replace(array_keys($variables), array_values($variables), $fileName));
            }

            if (isset($basenameParts[2]) && strpos($basenameParts[2], 'thumb') === 0) {
                $filePath = sprintf('%s/thumb/%s', $dirName, $fileName);
            } else {
                $filePath = sprintf('%s/%s', $dirName, $fileName);
            }
        }

        return $filePath;
    }

    /**
     * @param string $fileName
     * @return string
     */
    public static function getSafeFileName($fileName)
    {
        $fileName = utf8_romanize(utf8_deaccent($fileName));
        $fileName = strval(str_replace(' ', '_', $fileName));
        $fileName = strval(preg_replace('/[^a-z0-9._-]/i', '', $fileName));

        return $fileName;
    }
}
