<?php

namespace Xfrocks\AttachmentStore\Util;

use Xfrocks\AttachmentStore\Listener;

class MimeType extends \League\Flysystem\Util\MimeType
{
    /**
     * Guess MIME type by file name.
     * This method returns 'binary/octet-stream' if result is unknown.
     * Which is different from the parent method (returning 'text/plain').
     *
     * @param string $fileName
     * @return string
     *
     * @see \League\Flysystem\Util\MimeType::detectByFilename
     */
    public static function guessByFileName($fileName)
    {
        $extension = strtolower(pathinfo($fileName, PATHINFO_EXTENSION));

        $customMimeTypes = \XF::config(Listener::CONFIG_MIME_TYPES);
        if (is_array($customMimeTypes) && isset($customMimeTypes[$extension])) {
            return $customMimeTypes[$extension];
        }

        return isset(static::$extensionToMimeTypeMap[$extension])
            ? static::$extensionToMimeTypeMap[$extension]
            : 'binary/octet-stream';
    }
}
