<?php

namespace Xfrocks\AttachmentStore\XF\Entity;

use XF\Mvc\Entity\Structure;
use Xfrocks\AttachmentStore\Data\Storage;

/**
 * @property string bdattachmentstore_engine
 * @property array bdattachmentstore_options
 */
class AttachmentData extends XFCP_AttachmentData
{
    const XFROCKS_ATTACHMENTSTORE_OPTION_IS_NEWLY_INSERTED = 'Xfrocks_AttachmentStore_isNewlyInserted';
    const XFROCKS_ATTACHMENTSTORE_OPTION_SHOULD_BE_DELAYED = 'Xfrocks_AttachmentStore_shouldBeDelayed';

    /**
     * @return string
     */
    public function getAbstractedDataPath()
    {
        $dataPath = parent::getAbstractedDataPath();

        if (strlen($this->bdattachmentstore_engine) > 0) {
            /** @var Storage $storage */
            $storage = $this->app()->data('Xfrocks\AttachmentStore:Storage');
            $filePath = $storage->getFilePathData($this);
            return $storage->getAbstractedPathReplacement($this, $filePath, $dataPath);
        }

        return $dataPath;
    }

    /**
     * @return string
     */
    public function getAbstractedThumbnailPath()
    {
        $thumbnailPath = parent::getAbstractedThumbnailPath();

        if (strlen($this->bdattachmentstore_engine) > 0) {
            /** @var Storage $storage */
            $storage = $this->app()->data('Xfrocks\AttachmentStore:Storage');
            $filePath = $storage->getFilePathThumbnail($this);
            return $storage->getAbstractedPathReplacement($this, $filePath, $thumbnailPath);
        }

        return $thumbnailPath;
    }

    /**
     * @param mixed $canonical
     * @return string|null
     */
    public function getThumbnailUrl($canonical = false)
    {
        $thumbnailUrl = parent::getThumbnailUrl($canonical);

        if (!is_string($thumbnailUrl) || strlen($this->bdattachmentstore_engine) === 0) {
            return $thumbnailUrl;
        }

        /** @var Storage $storage */
        $storage = $this->app()->data('Xfrocks\AttachmentStore:Storage');
        $filePath = $storage->getFilePathThumbnail($this);
        return $storage->getFileUrlWithAttachmentData($this, $filePath);
    }

    /**
     * @param mixed $canonical
     * @return mixed|null
     */
    public function getVideoUrl($canonical = false)
    {
        $videoUrl = parent::getVideoUrl($canonical);

        if (!is_string($videoUrl) || strlen($this->bdattachmentstore_engine) === 0) {
            return $videoUrl;
        }

        /** @var Storage $storage */
        $storage = $this->app()->data('Xfrocks\AttachmentStore:Storage');
        $filePath = $storage->getFilePathData($this);
        return $storage->getFileUrlWithAttachmentData($this, $filePath);
    }

    protected function _preSave()
    {
        parent::_preSave();

        if (strlen($this->bdattachmentstore_engine) > 0) {
            return;
        }

        /** @var Storage $storage */
        $storage = $this->app()->data('Xfrocks\AttachmentStore:Storage');

        if (!$storage->isIgnored($this)) {
            $this->bdattachmentstore_engine = $storage->getDefaultEngine();
        }

        if (strlen($this->bdattachmentstore_engine) > 0 && $storage->shouldBeDelayed($this)) {
            $this->setOption(self::XFROCKS_ATTACHMENTSTORE_OPTION_SHOULD_BE_DELAYED, true);
            $this->bdattachmentstore_engine = '';
        }

        if (strlen($this->bdattachmentstore_engine) > 0) {
            $this->bdattachmentstore_options = $storage->getDefaultOptions($this->bdattachmentstore_engine, $this);
        }
    }

    protected function _postSave()
    {
        parent::_postSave();

        if ($this->isInsert()) {
            $this->setOption(self::XFROCKS_ATTACHMENTSTORE_OPTION_IS_NEWLY_INSERTED, true);
        }

        if ($this->getOption(self::XFROCKS_ATTACHMENTSTORE_OPTION_SHOULD_BE_DELAYED) === true) {
            /** @var Storage $storage */
            $storage = $this->app()->data('Xfrocks\AttachmentStore:Storage');
            $storage->delayQueue($this->db())->queue($this);
        }
    }

    public static function getStructure(Structure $structure)
    {
        $structure = parent::getStructure($structure);

        $structure->columns['bdattachmentstore_engine'] = ['type' => self::STR];
        $structure->columns['bdattachmentstore_options'] = ['type' => self::SERIALIZED_ARRAY];

        $structure->options[self::XFROCKS_ATTACHMENTSTORE_OPTION_IS_NEWLY_INSERTED] = false;
        $structure->options[self::XFROCKS_ATTACHMENTSTORE_OPTION_SHOULD_BE_DELAYED] = false;

        return $structure;
    }
}
